import base64
import datetime
import hashlib
import hmac
import os
import requests  # pip install requests
import sys
import time
# Debuging
import logging
import http.client as http_client
http_client.HTTPConnection.debuglevel = 1

# ************* REQUEST VALUES *************
method = 'POST'
service = 'api'
region = 'rc'
host = ".".join([service, region, "antavo.com"])
# e.g., host = 'api.rc.antavo.com'
canonical_uri = '/events'
canonical_querystring = ''
endpoint = "https://" + host + canonical_uri + canonical_querystring

# Make sure that the canonical_uri and canonical_querystring match the endpoint
# e.g., endpoint = 'https://api.rc.antavo.com/events'
content_type = 'application/json'
request_parameters =  '{'
request_parameters += '"customer":"1234567",'
request_parameters += '"action":"opt_in",'
request_parameters += '"data":{"email":"example@test.com"}'
request_parameters +=  '}'

# Create a date for headers and the credential string
t = datetime.datetime.utcnow()
date = t.strftime('%Y%m%dT%H%M%SZ')
date_stamp = t.strftime('%Y%m%d')  # Date w/o time, used in credential scope

# Read AWS access key from env. variables or configuration file. Best practice is NOT
# to embed credentials in code.
access_key = '**************'
secret_key = '**************'

# Key derivation functions. See:
# http://docs.aws.amazon.com/general/latest/gr/signature-v4-examples.html#signature-v4-examples-python


def sign(key, msg):
    return hmac.new(key, msg.encode("utf-8"), hashlib.sha256).digest()


def getSignatureKey(key, date_stamp, regionName, serviceName):
    kDate = sign(('ANTAVO' + key).encode('utf-8'), date_stamp)
    kRegion = sign(kDate, regionName)
    kService = sign(kRegion, serviceName)
    kSigning = sign(kService, 'antavo_request')
    return kSigning


# ************* TASK 1: CREATE A CANONICAL REQUEST *************
# http://docs.aws.amazon.com/general/latest/gr/sigv4-create-canonical-request.html

canonical_headers = 'date:' + date + '\n' + 'host:' + host + '\n'
signed_headers = 'date;host'

payload_hash = hashlib.sha256(request_parameters.encode('utf-8')).hexdigest()
canonical_request = method + '\n' + canonical_uri + '\n' + canonical_querystring + \
    '\n' + canonical_headers + '\n' + signed_headers + '\n' + payload_hash

# print("canonicalized_request - {}".format(canonical_request))
# ************* TASK 2: CREATE THE STRING TO SIGN*************
# Match the algorithm to the hashing algorithm you use, either SHA-1 or
# SHA-256 (recommended)
algorithm = 'ANTAVO-HMAC-SHA256'
credential_scope = date_stamp + '/' + region + \
    '/' + service + '/' + 'antavo_request'
string_to_sign = algorithm + '\n' + date + '\n' + credential_scope + \
    '\n' + hashlib.sha256(canonical_request.encode('utf-8')).hexdigest()
# print("string_to_sign - {}".format(string_to_sign))
# ************* TASK 3: CALCULATE THE SIGNATURE *************
# Create the signing key using the function defined above.
signing_key = getSignatureKey(secret_key, date_stamp, region, service)

# Sign the string_to_sign using the signing_key
signature = hmac.new(signing_key, (string_to_sign).encode(
    'utf-8'), hashlib.sha256).hexdigest()


# ************* TASK 4: ADD SIGNING INFORMATION TO THE REQUEST *************
# Put the signature information in a header named Authorization.
authorization_header = algorithm + ' ' + 'Credential=' + access_key + '/' + \
    credential_scope + ', ' + 'SignedHeaders=' + \
    signed_headers + ', ' + 'Signature=' + signature

# Python note: The 'host' header is added automatically by the Python 'requests' library.
headers = {
    'Content-Type': content_type,
    'date': date,
    'Authorization': authorization_header
}


# ************* SEND THE REQUEST *************
print('\nBEGIN REQUEST++++++++++++++++++++++++++++++++++++')
print('Request URL = ' + endpoint)
print('parameters')
print(request_parameters)

start = time.time()
r = requests.post(endpoint, data=request_parameters, headers=headers)
roundtrip = time.time() - start

print('\nRESPONSE++++++++++++++++++++++++++++++++++++')
print('Response code: %d\n' % r.status_code)
print(r.text)

print("Time taken - {}".format(r.elapsed.total_seconds()))
print("Time with content - {}".format(roundtrip))